/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/

//maximum number of objects (and allow user to shoot additional boxes)
#define MAX_PROXIES (ARRAY_SIZE_X*ARRAY_SIZE_Y*ARRAY_SIZE_Z + 1024)

#include "HACDDemo.h"
#include "GlutStuff.h"
///btBulletDynamicsCommon.h is the main Bullet include file, contains most common include files.
#include "btBulletDynamicsCommon.h"
#include <BulletCollision/Gimpact/btGImpactCollisionAlgorithm.h>
#include <BulletCollision/Gimpact/btGImpactShape.h>

#include "btCollisionShapeGarbageCollector.h"
#include "btHACDCompoundShape.h"
#include "btTriangleMeshEx.h"

#ifndef EXCLUDE_VHACD   //Tweakable but not here: in HACDDemo.h or in the 'compiler definitions' of the project
#include "btVHACDCompoundShape.h"
extern bool LoadVHACDCompoundShapeParams(char* paramsFilePath,btVHACDCompoundShape::Params& params,bool* extraParamUseAutoGeneratedSaveNameToSaveShape,bool* extraParamInvertYZ,bool* extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,bool* extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,bool* extraParamDecomposeBCSInputFileIfShapeIsCompatible,
                                         bool* extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape
                                         );
#endif //EXCLUDE_VHACD

extern ContactAddedCallback	gContactAddedCallback;

extern char gExeFileFullPath[1025];
extern char gFileFromCommandLine[1025];

extern bool FileExists(const char* filePath);
extern bool FileHasExtension(const char* filePath,const char* extensionString);
extern size_t GetExeFileDirectoryPlusRelativeFilePath(char* pBuffer, const size_t bufferSize,const char* exeFileFullPath,const char* relativeFilePath);
extern bool LoadHACDCompoundShapeParams(char* paramsFilePath,btHACDCompoundShape::Params& params,bool* extraParamUseAutoGeneratedSaveNameToSaveShape,bool* extraParamInvertYZ,bool* extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,bool* extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,bool* extraParamDecomposeBCSInputFileIfShapeIsCompatible,
                                        bool* extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape,bool* extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable);
static void DisplayAvgNumberOfHullVerticesInChildShapes(const btCompoundShape* cs);

#include <stdio.h> //printf debugging


//btCollisionShapeGarbageCollector m_collisionShapes;	// To greatly simplify the collision shape memory handling (see exitPhysics())
btAlignedObjectArray < btTriangleMeshEx* > gPersistentMeshes;	// Needed only if we want to pass them to btBvhTriangleMeshShape (and btScaledTriangleMeshShape) or btGImpactMeshShape.

// Much better materials can be built....
static Material gPersistentMaterials[] ={Material(1,0,0),Material(1,1,0),Material(0,1,0),Material(0,0,1),Material(1,0.45,0)};	// Used with optional stuff related to the 'r' key
static const int gNumPersistentMaterials = sizeof(gPersistentMaterials)/sizeof(gPersistentMaterials[0]); 

// Stuff needed by the SlidingSledgeDemo only: ---------------------------------------------------------
//#define USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO		// The sledge slides MUCH better!
//#define USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO	// The default implementation uses btRigidBody->setAnisotropicFriction(...): but this does not seem to be tweakable in
													// the custom material callback (AFAIK so far). So I did an alternative implementation, but it's probably much slower... 
													// and I'm not even sure it's correct!
const btCollisionShape* gSledgeShape = NULL;
btAlignedObjectArray < int > gSledgeShapeSubmeshIndexOfChildShapes;
#ifdef USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO	
const btVector3 gAnisotropicFrictionOLowerPartOfTheSledge(0.3,0.,0.);
#endif //USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO


inline static void GetFrictionRestitutionForTheSledgeShape(const btCollisionObject* colObj,int subMeshIndex,float& friction,float& restitution)	{
	if (subMeshIndex==0) {
		// ski part of the sledge (TODO: bidirectional friction is a must here...)
		friction = 0.;
		restitution = 0.;
		#ifdef USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		const btRigidBody* rb = btRigidBody::upcast(colObj);
		if (rb)	{
			btVector3 localVelocity = rb->getLinearVelocity() * rb->getWorldTransform().getBasis();	
			localVelocity.normalize();
			friction = 	btFabs(localVelocity.x()) * gAnisotropicFrictionOLowerPartOfTheSledge.x() + 
						btFabs(localVelocity.y()) * gAnisotropicFrictionOLowerPartOfTheSledge.y() + 
						(localVelocity.z()>0 ? btFabs(localVelocity.z()) * gAnisotropicFrictionOLowerPartOfTheSledge.z() : btFabs(localVelocity.z()) * gAnisotropicFrictionOLowerPartOfTheSledge.x()*0.2f);
		}			
		#endif //USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
	}	
	else	{
		// upper part of the sledge
		friction = 2.5;
		restitution = 1.5;					
	}	
}

inline btScalar	calculateCombinedFriction(float friction0,float friction1)	{
	btScalar friction = friction0 * friction1;

	const btScalar MAX_FRICTION  = 10.f;
	if (friction < -MAX_FRICTION)
		friction = -MAX_FRICTION;
	if (friction > MAX_FRICTION)
		friction = MAX_FRICTION;
	return friction;

}
inline btScalar	calculateCombinedRestitution(float restitution0,float restitution1)	{
	return restitution0 * restitution1;
}
///CustomMaterialCombinerCallback is just an example to show how to get access to the child shape that collided
//#define USE_AN_OLDER_BULLET_LIBRARY_VERSION //Tweakable in case of compilation errors
#ifndef USE_AN_OLDER_BULLET_LIBRARY_VERSION
static bool CustomMaterialCombinerCallback(btManifoldPoint& cp,	const btCollisionObjectWrapper* colObj0Wrapper,int partId0,int index0,const btCollisionObjectWrapper* colObj1Wrapper,int partId1,int index1)
{
    const btCollisionObject* colObj0 = colObj0Wrapper->getCollisionObject();
    const btCollisionObject* colObj1 = colObj1Wrapper->getCollisionObject();    

	float friction0 = colObj0->getFriction();
	float friction1 = colObj1->getFriction();
	float restitution0 = colObj0->getRestitution();
	float restitution1 = colObj1->getRestitution();

	if (colObj0->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)	{
        if (colObj0->getCollisionShape() == gSledgeShape)	{
			#ifndef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
			if (index0>=0 && index0<gSledgeShapeSubmeshIndexOfChildShapes.size())	GetFrictionRestitutionForTheSledgeShape(colObj0,gSledgeShapeSubmeshIndexOfChildShapes[index0],friction0,restitution0);
			#else //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
			GetFrictionRestitutionForTheSledgeShape(colObj0,partId0,friction0,restitution0);
			#endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		}
	}	
	if (colObj1->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)	{
        if (colObj1->getCollisionShape() == gSledgeShape)	{
			#ifndef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
			if(index1>=0 && index1<gSledgeShapeSubmeshIndexOfChildShapes.size())	GetFrictionRestitutionForTheSledgeShape(colObj1,gSledgeShapeSubmeshIndexOfChildShapes[index1],friction1,restitution1);
			#else //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
			GetFrictionRestitutionForTheSledgeShape(colObj1,partId1,friction1,restitution1);
			#endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		}	
	}	
	
	cp.m_combinedFriction = calculateCombinedFriction(friction0,friction1);
	cp.m_combinedRestitution = calculateCombinedRestitution(restitution0,restitution1);

	//this return value is currently ignored, but to be on the safe side: return false if you don't calculate friction
	return true;
}
#else //USE_AN_OLDER_BULLET_LIBRARY_VERSION
static bool CustomMaterialCombinerCallback(btManifoldPoint& cp,	const btCollisionObject* colObj0,int partId0,int index0,const btCollisionObject* colObj1,int partId1,int index1)
{

    float friction0 = colObj0->getFriction();
    float friction1 = colObj1->getFriction();
    float restitution0 = colObj0->getRestitution();
    float restitution1 = colObj1->getRestitution();

    if (colObj0->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)	{
        if (colObj0->getRootCollisionShape() == gSledgeShape)	{
            #ifndef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
            if (index0>=0 && index0<gSledgeShapeSubmeshIndexOfChildShapes.size())	GetFrictionRestitutionForTheSledgeShape(colObj0,gSledgeShapeSubmeshIndexOfChildShapes[index0],friction0,restitution0);
            #else //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
            GetFrictionRestitutionForTheSledgeShape(colObj0,partId0,friction0,restitution0);
            #endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
        }
    }
    if (colObj1->getCollisionFlags() & btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK)	{
        if (colObj1->getRootCollisionShape() == gSledgeShape)	{
            #ifndef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
            if(index1>=0 && index1<gSledgeShapeSubmeshIndexOfChildShapes.size())	GetFrictionRestitutionForTheSledgeShape(colObj1,gSledgeShapeSubmeshIndexOfChildShapes[index1],friction1,restitution1);
            #else //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
            GetFrictionRestitutionForTheSledgeShape(colObj1,partId1,friction1,restitution1);
            #endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
        }
    }

    cp.m_combinedFriction = calculateCombinedFriction(friction0,friction1);
    cp.m_combinedRestitution = calculateCombinedRestitution(restitution0,restitution1);

    //this return value is currently ignored, but to be on the safe side: return false if you don't calculate friction
    return true;
}
#endif //USE_AN_OLDER_BULLET_LIBRARY_VERSION
//--------------------------------------------------------------------------------------------------------

void HACDDemo::clientMoveAndDisplay()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//simple dynamics world doesn't handle fixed-time-stepping
	float ms = getDeltaTimeMicroseconds();
	
	///step the simulation
	if (m_dynamicsWorld)
	{
		m_dynamicsWorld->stepSimulation(ms / 1000000.f);
		//optional but useful: debug drawing
		m_dynamicsWorld->debugDrawWorld();
	}
		
	renderme(); 

	glFlush();

	swapBuffers();

}



void HACDDemo::displayCallback(void) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	
	renderme();

	//optional but useful: debug drawing to detect problems
	if (m_dynamicsWorld)
		m_dynamicsWorld->debugDrawWorld();
		
	glFlush();
	swapBuffers();
}

void HACDDemo::myinit(void)
{
        GlutDemoApplicationEx::myinit();
        /*
        //--------------------------
        // DemoApplication::myinit():
        //----------------------------
        GLfloat light_ambient[] = { btScalar(0.2), btScalar(0.2), btScalar(0.2), btScalar(1.0) };
        GLfloat light_diffuse[] = { btScalar(1.0), btScalar(1.0), btScalar(1.0), btScalar(1.0) };
        GLfloat light_specular[] = { btScalar(1.0), btScalar(1.0), btScalar(1.0), btScalar(1.0 )};
        //	light_position is NOT default value
        GLfloat light_position0[] = { btScalar(1.0), btScalar(10.0), btScalar(1.0), btScalar(0.0 )};
        GLfloat light_position1[] = { btScalar(-1.0), btScalar(-10.0), btScalar(-1.0), btScalar(0.0) };

        glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
        glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
        glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
        glLightfv(GL_LIGHT0, GL_POSITION, light_position0);

        glLightfv(GL_LIGHT1, GL_AMBIENT, light_ambient);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, light_diffuse);
        glLightfv(GL_LIGHT1, GL_SPECULAR, light_specular);
        glLightfv(GL_LIGHT1, GL_POSITION, light_position1);

        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1);    // 2 lights are active:maybe we can switch this one off for a small speed up.


        glShadeModel(GL_SMOOTH);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS);

        glEnable(GL_CULL_FACE);
        glCullFace(GL_BACK);
        //-----------------------------
        // Plus: GlutDemoApplicationEx::myinit():
        //-----------------------------
        glClearColor(btScalar(0.6),btScalar(0.8),btScalar(1.0),btScalar(1));
        glDisable(GL_LIGHT1);
        */
		glEnable(GL_LIGHT1);	
}


void 	HACDDemo::createEmptyDynamicsWorld()	{

	m_azi = 180;m_ele = 20;
	setCameraDistance(btScalar(45.));	
	// AlternativeNavigationSystem is free camera mode	
	if (isAlternativeNavigationSystemEnabled()) 
	{
		//We want to force m_azi and m_ele and camera distance to generate a m_cameraT (instead of using the optional params passed to the ctr)
		// This is the only case in which the 2nd arg of this method must be used:
		this->setAlternativeNavigationSystemEnabled(true,true);
	}
	m_frustumZNear = 0.1f;
	m_frustumZFar = 5000.f;

	GlutDemoApplicationEx::createEmptyDynamicsWorld(); // GImpact collision algorithm and ghost objects are activated inside it AFAIR
	// If somebody wants to create his own world instead, he can reuse the variable already stored in GlutDemoApplicationEx 
	// (m_collisionConfiguration, m_dispatcher, m_dispatcher, m_solver, m_dynamicsWorld). No need to redefine them in "HACDDemo.h".

	
}

void	HACDDemo::createGround(btScalar groundHalfDimension)	{
	///create a few basic rigid bodies
	btCollisionShape* groundShape = new btBoxShape(btVector3(groundHalfDimension,groundHalfDimension,groundHalfDimension));
	//m_collisionShapes.push_back(groundShape);	// Not needed anymore, but still optional

	// Optional Stuff ('r' key) 
	{
		char textureFullPath[1025];	// "appHACDDemoMeshes/ground.jpg"
		{
			// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
			// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
			// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
			textureFullPath[0]='\0';	
                        GetExeFileDirectoryPlusRelativeFilePath(textureFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/ground.jpg");
		}
		const GLuint groundTexture = TextureHelper::Get(textureFullPath);
		if (groundTexture>0)	{
			btTriangleMeshEx mesh;
			const btScalar texRep(groundHalfDimension*btScalar(0.25));
			mesh.loadDefaultPlaneMesh(2,groundHalfDimension,groundHalfDimension,texRep,texRep);//,0,0,true);
			
			Material* mat = mesh.getMeshSubPartMaterial(0);
			if (mat) {
				// We modify it a bit...
				//mat->amb[0]=mat->amb[1]=mat->amb[2]=0.4f;
				//mat->dif[0]=mat->dif[1]=mat->dif[2]=1.0f;				
				mat->spe[0]=mat->spe[1]=mat->spe[2]=0.0f;
				mat->shi = 0.0f;
			}
            //mesh.setMeshSubPartTextureId(0,groundTexture);	// Here we set our groud texture for the mesh, but doing so
            													// overrides can only change it, not remove it.
			
                        this->AssignMesh(groundShape,mesh,true);    // last arg = use flat normals
						GetMeshOverrides(*groundShape).offset = btVector3(0,groundHalfDimension,0);	// This is because we are assigning a planar mesh to a cube
            			GetMeshOverrides(*groundShape).textureArray.push_back(groundTexture);   // (or we can directly set it to 'mesh' 3 lines above. This approach leaves the original mesh without textures assigned, so that shapes can have no texture if needed).
                        //mesh.displayDebugInfo();
			//printf("GROUND SHOULD BE OK\n");
		}
                //else printf("GROUND NOT OK. TEXTURE NOT FOUND %s.\n",textureFullPath);
	}

	btTransform groundTransform;
	groundTransform.setIdentity();
	groundTransform.setOrigin(btVector3(0,-groundHalfDimension,0));

	//We can also use DemoApplication::localCreateRigidBody, but for clarity it is provided here:
	{
		btScalar mass(0.);

		//rigidbody is dynamic if and only if mass is non zero, otherwise static
		bool isDynamic = (mass != 0.f);

		btVector3 localInertia(0,0,0);
		if (isDynamic)
			groundShape->calculateLocalInertia(mass,localInertia);

		//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
		btDefaultMotionState* myMotionState = new btDefaultMotionState(groundTransform);
		btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,groundShape,localInertia);
		btRigidBody* body = new btRigidBody(rbInfo);

		body->setFriction(0.5f);
		body->setRestitution(0.5f);
		
		//add the body to the dynamics world
		m_dynamicsWorld->addRigidBody(body);
	}	
}
	
#include <BulletCollision/CollisionShapes/btConvexPolyhedron.h>
void	HACDDemo::initPhysics()	{
    myinit();    // MANDATORY: Sets up lights and gl stuff, can be overridden
	m_debugMode |= btIDebugDraw::DBG_NoHelpText;
	createEmptyDynamicsWorld();

	if (gFileFromCommandLine[0]!='\0' && m_demoNumber==-1)	{
		initPhysicForCommandlineUsage();		
		return;
	}
	if (m_demoNumber<0) m_demoNumber=0;
		
	
	switch (m_demoNumber%numDemos)	{
		default:
		case 0:	initPhysicsForDemoBunnyFamily();		// Basic usage tutorial
				break;
		case 1: initPhysicsForDemoTorusEvolution();		// HACD params tutorial
				break;	
		case 2: initPhysicsForDemoTablesAndChairs();	// Serialization tutorial
				break;					
		case 3: initPhysicsForDemoSlidingSledge();		// Multimaterial tutorial
                break;
        #ifndef EXCLUDE_VHACD
        case 4:	initPhysicsForDemoVHACDTest();          // VHACD test
        break;
        #endif
		}
	

}
	
// Basic usage tutorial
void	HACDDemo::initPhysicsForDemoBunnyFamily()	{
		printf("\nStarting 'BunnyFamilyDemo'. Please wait...\n");

		createGround();

		btTriangleMeshEx mesh;			// Our mesh class. Compound shapes don't require it to be persistent (so we scope it).
		#ifdef INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
		mesh.loadDefaultBunnyMesh();	// Other loading methods are available.
		#else //INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
		{
			char modelFullPath[1025];	// "appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultBunnyMesh.off"
			// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
			// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
			// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
			modelFullPath[0]='\0';	
                        GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultBunnyMesh.off");
			if (!mesh.loadFromFile(modelFullPath))	{
				printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
				return;
			}
		}
		#endif //INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
		
		//mesh.center();	// This centers the mesh (i.e. will shift the center of mass to the center).
		mesh.setHeight(8);	// in meters! This scales the mesh in an uniform way so that the height is selected.
					
		btCompoundShape* shape = new btHACDCompoundShape(mesh);	// This might take a while
		
		mesh.setMeshSubPartMaterial(0,gPersistentMaterials[0]);	// The bunny mesh will be white without this line... (actually materials don't have to be persistent here...)
		this->AssignMesh(shape,mesh);	// New: Optional Stuff ('r' key') -- creates a series of display lists, so that the mesh can go safely out of scope later, and stuffs it into the shape user pointer
			
		btScalar mass(1.);
		btTransform T; T.setIdentity(); T.setOrigin(btVector3(0,10,0));
			
		// create rigid body and add it to the world (ALL replaceble by: localCreateRigidBody(mass,T,shape);)
		{
				//rigidbody is dynamic if and only if mass is non zero, otherwise static
				bool isDynamic = (mass != 0.f);
				btVector3 localInertia(0,0,0);
				if (isDynamic) shape->calculateLocalInertia(mass,localInertia);
				//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
				btDefaultMotionState* myMotionState = new btDefaultMotionState(T);
				btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,shape,localInertia);
				btRigidBody* body = new btRigidBody(rbInfo);
				
				//add the body to the dynamics world
				m_dynamicsWorld->addRigidBody(body);
		}
			
		
		// OK. So far, so good. Now we add a row with scaled shapes:
		
			T.setIdentity(); T.setOrigin(btVector3(-50,10,-20));
						
			for (int i = 1;i<5;i++)	{
				const btScalar uniformScalingFactor = 0.5 * i;
				btCollisionShape* cs = btUtils::GetUniformScalingCompoundShape(shape,uniformScalingFactor);
				
				// Optional Stuff - 'r' key
				{
                    //AssignMesh(cs,mesh,false,btVector3(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor));	// Deprecated: this creates a brand new series of display lists for the same mesh ('r' key') (bool arg = useFlatNormals)
					AssignMeshCopy(cs,shape);	// Cheaper: the display lists are shared, but we must still override the mesh scaling
					btTriangleMeshExFootPrint::Overrides& ovrs = GetMeshOverrides(*cs);	
					ovrs.scaling = btVector3(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor);	// needed to override the mesh scaling
					ovrs.materialArray.push_back(&gPersistentMaterials[i%gNumPersistentMaterials]);	//color override: 1st material (and unique for this mesh)
					// VERY IMPORTANT: When we override a material inside any 'Mesh Overrides' it MUST be persistent (that's the only important thing to remember)!!!
				}
				
				const btScalar mass(uniformScalingFactor*3);
				T.getOrigin().setX(T.getOrigin().x()+20);
			
				if (i<4)	/*btRigidBody* body = */localCreateRigidBody(mass,T,cs);	// Equivalent to the code above (INCLUDING addRigidBody())	
				else	{
					// We add a center of mass (COM) offset to the last body:
					const btVector3 curBodyHalfExtents = btUtils::GetAabbHalfExtents(shape)*uniformScalingFactor;
					const btVector3 comOffset(0,-curBodyHalfExtents.y()*btScalar(3.0/4.0),-curBodyHalfExtents.z()*btScalar(2.0/3.0));
					btRigidBody* body = localCreateRigidBodyWithCOMOffset(mass,T,cs,comOffset);	// Equivalent to the code above (INCLUDING addRigidBody())
			
					// Debug Stuff (please check if it's correct)
					/*
					{
						btDefaultMotionState* ms = (btDefaultMotionState*) body->getMotionState();
						if (ms)	{
							btTransform T;
							ms->getWorldTransform(T);
							printf("World Position Of COM:		(%1.2f,%1.2f,%1.2f)\n",T.getOrigin().x(),T.getOrigin().y(),T.getOrigin().z());
						}
					}	
					#ifdef USE_COLLISION_OBJECT_USER_POINTER 	// defined in "GlutDemoApplicationEx.h": allows people to retrieve the transform of the body "outside" the center of mass.
																// when defined btCollisionObject::get/setUserPointer() cannot be used anymore, but can be replaced by this->Get/SetUserPointer(...)
					{
						btMyDefaultMotionState* ms = btMyDefaultMotionState::upcast(body);
						if (ms)	{
							btTransform T;
							ms->getWorldTransformOutsideCOM(T);
							printf("World Position Outside COM:	(%1.2f,%1.2f,%1.2f)\n",T.getOrigin().x(),T.getOrigin().y(),T.getOrigin().z());
						}
					}
					#endif //USE_COLLISION_OBJECT_USER_POINTER					
					*/
				}					
			}

		// All very easy so far, isn't it ?

	printf("'BunnyFamilyDemo' ready.\n");
	printf("Press '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");	
}
// HACD params tutorial
void	HACDDemo::initPhysicsForDemoTorusEvolution()	{
	printf("\nStarting 'TorusEvolutionDemo'. Please wait...\n");
	
	createGround();

	btTriangleMeshEx* pmesh = new btTriangleMeshEx();
	gPersistentMeshes.push_back(pmesh);
	btTriangleMeshEx& mesh = *pmesh;		// Now the mesh is persistent (needed because we'll pass it to a btBvhTriangleMeshShape later)

	#ifdef INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
	mesh.loadDefaultTorusMesh();	// Other loading methods are available.
	#else //INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
	{
		char modelFullPath[1025];	// "appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultTorusMesh.off"
		// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
		// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
		modelFullPath[0]='\0';	
		GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultTorusMesh.off");		
		if (!mesh.loadFromFile(modelFullPath))	{
			printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
			return;
		}
	}
	#endif //INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO	

	mesh.center();	// This centers the mesh (i.e. will shift the center of mass to the center).
	mesh.transform(btTransform(btQuaternion(btVector3(1,0,0),SIMD_HALF_PI),btVector3(0,0,0)));	//We rotate the mesh 90 degree around the X axis
	mesh.setHeight(8);	// in meters! This scales the mesh in an uniform way so that the height is selected.

	btCollisionShape* shape;
			
	btScalar mass(0.);
	btTransform T; T.setIdentity(); 
	
	{
		shape = new btHACDCompoundShape(mesh);	// This might take a while
		T.setOrigin(btVector3(0,4,0));
		localCreateRigidBody(mass,T,shape);
		
		mesh.setMeshSubPartMaterial(0,gPersistentMaterials[0]);	// The torus mesh will be white without this line...
		this->AssignMesh(shape,mesh);	// Optional Stuff ('r' key') -- creates a series of display lists, so that the mesh can go safely out of scope later, and stuffs it into the shape user pointer
	}
	btCollisionShape* firstShape = shape;
	
	
	// Two problems:
	// 1) the torus hole has been filled	
	// 2) the collision shape is too coarse	
	
	btHACDCompoundShape::Params params;
	params.addExtraDistPoints = params.addFacesPoints = true;	// This will handle holes in concave meshes better
	//params.displayDebugInfo = true;	
	{
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		T.setOrigin(btVector3(-30,4,-20));
		localCreateRigidBody(mass,T,shape);	
		
		this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
		this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[1%gNumPersistentMaterials]); // we color this shape in a different way
	}	
	
	//PS. It should be better to call params.reset() before each new use, but I set the params incrementally...
	params.nClusters = 8;	// (1) Min number of resulting sub shapes. Can be used to increase the number of sub shapes.
	
	{
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		T.setOrigin(btVector3(-10,4,-20));
		localCreateRigidBody(mass,T,shape);	
		
		this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
		this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[2%gNumPersistentMaterials]); // we color this shape in a different way
	}	
	
	params.nClusters = 12;			// (1) Min number of resulting sub shapes. Can be used to increase the number of sub shapes.
	params.maxHullVertices = 64;	// (16) Max number of vertices in each convex hull sub shape. (This will improve the "normals" of the shapes)
	
	{
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		T.setOrigin(btVector3(10,4,-20));
		localCreateRigidBody(mass,T,shape);	
		
		this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
		this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[3%gNumPersistentMaterials]); // we color this shape in a different way
	}	
	
	// We can still improve the shape playing around with these values.
	
	// The state of the art shape (for comparison) is the one without a shadow:
	{
		shape = new btBvhTriangleMeshShape(mesh,true);	// For comparison (needs persistent mesh)
		T.setOrigin(btVector3(30,4,-20));
		localCreateRigidBody(0.,T,shape);		
		
		this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
		this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[4%gNumPersistentMaterials]); // we color this shape in a different way

		// The following is not needed in most cases, but on the "Bullet Forum" somebody asked how to decompose a btBvhTriangleMeshShape directly:
		//#define TEST_BVH_TO_HACD
		#ifdef TEST_BVH_TO_HACD
		{
			#ifndef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE	// This is defined in "btTriangleMeshEx.h", but can be disabled
			{
			// Direct conversion: fastest (ang independent on the LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE definition).
			shape = new btHACDCompoundShape(shape,params);	
			T.setOrigin(btVector3(50,4,-20));
			localCreateRigidBody(0.,T,shape);		
			
			// However can assign it only a previously loaded shape:
			this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
			this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[5%gNumPersistentMaterials]); // we color this shape in a different way
			}
			#else //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
			{
			// Here we test an alternative (much slower) way:
			btTriangleMeshEx mesh;
			if (mesh.loadFromMeshInterfaceInsideBulletCollisionShape(shape))	{
				//mesh.displayDebugInfo();
				shape = new btHACDCompoundShape(mesh,params);	
				T.setOrigin(btVector3(50,4,-20));
				localCreateRigidBody(0.,T,shape);		
				this->AssignMesh(shape,mesh);	// Optional Stuff ('r' key')
			}
			else printf("TEST_BVH_TO_HACD with LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE failed.\n");
			}
			#endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
			
			T.setOrigin(btVector3(50,4,-20));
			localCreateRigidBody(0.,T,shape);		
		
			this->AssignMeshCopy(shape,firstShape);	// Optional Stuff ('r' key') -- copies the series of display lists in firstShape user pointer inside shape user pointer
			this->GetMeshOverrides(*shape).materialArray.push_back(&gPersistentMaterials[5%gNumPersistentMaterials]); // we color this shape in a different way
			
		}
		#endif //TEST_BVH_TO_HACD
		
	}

	printf("'TorusEvolutionDemo' ready.\n");
	printf("Press '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");
}
// Serialization tutorial
void	HACDDemo::initPhysicsForDemoTablesAndChairs()	{
	printf("\nStarting 'TableAndChairsDemo'. Please wait...\n");

	createGround();
	
	btTriangleMeshEx  mesh;
	
	//=======
	// TABLE
	//=======
	
	char modelFullPath[1025];	// "appHACDDemoMeshes/table.obj"
	{
		// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
		// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
		modelFullPath[0]='\0';	
		GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/table.obj");
	}	
		
	if (!mesh.loadFromFile(modelFullPath))	{
		printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
		return;
	}
	//else mesh.saveToOFFFile("MyTable.off");


	//mesh.center();	// This centers the mesh (i.e. will shift the center of mass to the center).
	mesh.setHeight(4);	// in meters! This scales the mesh in an uniform way so that the height is selected. BTW: the scale in all this demo is too big. No time to fix it...

	btCollisionShape* shape;
			
	btScalar mass(5.);
	btTransform T; T.setIdentity(); 

	
	
	{
		btHACDCompoundShape::Params params;
		params.nClusters = 5;	// (1) Min number of resulting sub shapes. We exclude the possibility of a too low number of convex hulls for the whole shape	
		params.concavity = 200;	// (200) (This controls the number of hulls too) lower values => more shapes (and slower runtime performance)
        //params.displayDebugInfo = true;
		
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		T.setOrigin(btVector3(0,mesh.getAabbFullExtents().y(),0));
		localCreateRigidBody(mass,T,shape);

                this->AssignMesh(shape,mesh,true);	// Optional Stuff ('r' key') -- creates a series of display lists, so that the mesh can go safely out of scope later, and stuffs it into the shape user pointer
	}
	
	//=======
	//  CHAIR
	//=======
	mass = 1.;
	
	mesh.reset();	// Multiple meshes can be loaded together otherwise
	
	// Set modelFullPath to point to: "appHACDDemoMeshes/chair.obj", but global path (more robust, read below)
	{
		// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
		// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
		modelFullPath[0]='\0';	
		GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/chair.obj");
	}	
		
	if (!mesh.loadFromFile(modelFullPath))	{
		printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
		return;
	}		
	//else mesh.saveToOFFFile("MyChair.off");
	
	btHACDCompoundShape::Params params;
	params.nClusters = 2;	// (1) Min number of resulting sub shapes. We exclude the possibility of a too low number of convex hulls for the whole shape	
	params.concavity = 100;	// (200) (This controls the number of hulls too) lower values => more shapes (and slower runtime performance)
	params.addExtraDistPoints =  params.addFacesPoints = true;	// This will handle holes in concave meshes better (our chair has one hole in it)
	params.optionalBCSSaveFilePath = "appHACDDemoMeshes/chair.obj.bcs";	// Will be serialized to it
    //params.displayDebugInfo = true;
	{
		// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
		// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
		char temp[105];
		temp[0]='\0';	
		GetExeFileDirectoryPlusRelativeFilePath(temp,1024,gExeFileFullPath,params.optionalBCSSaveFilePath.c_str());
		params.optionalBCSSaveFilePath = temp;
	}

	shape = NULL;
	//#define DONT_CREATE_JUST_LOAD_SHAPE
	//#define DONT_LOAD_JUST_CREATE_SHAPE
	#ifndef DONT_CREATE_JUST_LOAD_SHAPE
	{	
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		#ifndef DONT_LOAD_JUST_CREATE_SHAPE
		delete shape; shape = NULL;	// We don't use it right now, we want to load it back instead.
		#endif //DONT_LOAD_JUST_CREATE_SHAPE
	}
	#endif //DONT_CREATE_JUST_LOAD_SHAPE
	
	#ifndef DONT_LOAD_JUST_CREATE_SHAPE
	shape = btUtils::Load(params.optionalBCSSaveFilePath.c_str());
	#endif //DONT_LOAD_JUST_CREATE_SHAPE
	if (!shape) {
		printf("ERROR: Can't load \"%s\" back.\n",params.optionalBCSSaveFilePath.c_str());
		return;
	}	
	else printf("Bullet Collision Shape saved to \"%s\" and loaded back successfully.\n",params.optionalBCSSaveFilePath.c_str());
	
        this->AssignMesh(shape,mesh,true);	// Optional Stuff ('r' key') -- creates a series of display lists, so that the mesh can go safely out of scope later, and stuffs it into the shape user pointer
	
	const btScalar yPos = mesh.getAabbFullExtents().y();
	for (int i = 1;i<4;i++)
	{
		T.setOrigin(btVector3((5*i-10)*0.5,yPos,2.5));
		localCreateRigidBody(mass,T,shape);
	}
	T.setRotation(btQuaternion(btVector3(0,1,0),SIMD_PI));
	for (int i = 1;i<4;i++)
	{
		T.setOrigin(btVector3((5*i-10)*0.5,yPos,-2.5));
		localCreateRigidBody(mass,T,shape);
	}
	
	#ifdef USE_COLLISION_OBJECT_USER_POINTER  	// defined in "GlutDemoApplicationEx.h"
												// when defined btCollisionObject::get/setUserPointer() cannot be used anymore, but can be replaced by this->Get/SetUserPointer(...)
	// Optional Stuff: ('r' key and the def above defined)
	{
		// We have seen so far how to share or build a mesh footprint (i.e. a series of display lists with overridable materials and textures) for btCollisionShape user pointers.
		// But what if I want to reuse the same exact collision shape and I want to override materials on a per body basis ?
		// Well the API is very similiar, but for now let's think about how to position the last two chairs...
		T.setRotation(btQuaternion(btVector3(0,1,0),SIMD_HALF_PI));
		T.setOrigin(btVector3(4.5f,yPos,0));
		btRigidBody* body = localCreateRigidBody(mass,T,shape);
		this->AssignMeshCopy(body,shape);	// Here it is the new API
		// Now we must decide what material to use. chair.obj has a single (sub)mesh with a texture and a material tha is not fully "white". We'd like to lighten it up just a bit:
		static Material myStaticMaterial;	// Material overrides must be persistent.
		myStaticMaterial = *mesh.getMeshSubPartMaterial(0);
		myStaticMaterial.lighten(0.5f);	// We could change the diffuse color too, in case it's more evident (without shadows ('g' key) it's evident)
		this->GetMeshOverrides(*body).materialArray.push_back(&myStaticMaterial);
		
		// Now the other chair:
		T.setRotation(btQuaternion(btVector3(0,1,0),-SIMD_HALF_PI));
		T.setOrigin(btVector3(-4.5f,yPos,0));
		btRigidBody* body2 = localCreateRigidBody(mass,T,shape);
		this->AssignMeshCopy(body2,body);	// Here it is the new API
				
		//The drawback is that GlutDemoApplicationEx uses the btCollisionObject user ptr...	but users should be able to use SetUserPointer(...)/GetUserPointer(...) instead.
	}
	#endif //USE_COLLISION_OBJECT_USER_POINTER	
		
	params.reset();	// can be optionally used to reset all params to their default values.

	printf("'TableAndChairsDemo' ready.\n");
	printf("Press '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");
}
// Multimaterial tutorial
void	HACDDemo::initPhysicsForDemoSlidingSledge()	{
	printf("\nStarting 'SlidingSledgeDemo'. Please wait...\n");
		
        createGround();
        // Optional Stuff ('r' key): we want the ground to be without texture with a white material----------------
        static const Material snowMaterial(0.8,0.8,0.7,false,0.3);  // Must be static. (Actually it doesn't seem snow at all, but at least it isn't too sparkling)
        btCollisionShape* originalGroundShape = NULL;
        if (this->m_dynamicsWorld->getCollisionObjectArray().size()>0)  {
            originalGroundShape = this->m_dynamicsWorld->getCollisionObjectArray()[0]->getCollisionShape();
            if (originalGroundShape)    {
                GetMeshOverrides(*originalGroundShape).textureArray.resize(0);	// This removes the ground texture that was set as override in createGround(). (Having a snow texture would be better)
                GetMeshOverrides(*originalGroundShape).materialArray.push_back(&snowMaterial);
            }
        }
        //----------------------------------------------------------------------------------------------------------

	//m_dynamicsWorld->getSolverInfo().m_splitImpulse = true;	// Optional...
	
	
	// MANDATORY:
	//============
	gContactAddedCallback = CustomMaterialCombinerCallback;
	
	//============
	// BACKGROUND
	//============
	const btScalar groundHalfDimension(100.);
	{
		btCollisionShape* groundShape = new btBoxShape(btVector3(groundHalfDimension*0.5f,10.0f,groundHalfDimension));
                // Optional Stuff ('r' key)---------------------------------------------------------------------------
                if (originalGroundShape)    {
                    this->AssignMeshCopy(groundShape,originalGroundShape);  // This method copies both the mesh 'footprint' AND the mesh overrides.
                    // However the mesh assigned to 'originalGroundShape' was a btBoxShape(btVector3(groundHalfDimension,groundHalfDimension,groundHalfDimension)), so:
                    const btVector3 additionalScaling = btVector3(0.5f,0.1f,1.0f);
                    GetMeshOverrides(*groundShape).scaling*= additionalScaling; // original scaling was zero, so we could just use '=' instead of '*='
                    GetMeshOverrides(*groundShape).offset*= additionalScaling;  // this can be a bit more difficult to understand..., but the original offset (due to the fact that a plane mesh was assigned to a box shape)
                                                                                // was calibrated on the original ground shape (vertical half distance to the box center); thus now that distance must be updated.
                }
                //----------------------------------------------------------------------------------------------------

                btRigidBody* body = NULL;
		btTransform groundTransform;
		groundTransform.setIdentity();
	
		groundTransform.setOrigin(btVector3(-groundHalfDimension,5.0f,0));
		groundTransform.setRotation(btQuaternion(btVector3(0,0,1),btRadians(-30)));
		body = localCreateRigidBody(0.,groundTransform,groundShape);
		body->setFriction(0.5f);
		body->setRestitution(0.5f);
		
		groundTransform.setOrigin(btVector3(groundHalfDimension,5.0f,0));
		groundTransform.setRotation(btQuaternion(btVector3(0,0,1),btRadians(30)));
		body = localCreateRigidBody(0.,groundTransform,groundShape);
		body->setFriction(0.5f);
		body->setRestitution(0.5f);		

		groundShape = new btBoxShape(btVector3(groundHalfDimension,10.0f,groundHalfDimension*0.5f));	
                // Optional Stuff ('r' key)---------------------------------------------------------------------------
                if (originalGroundShape)    {
                    this->AssignMeshCopy(groundShape,originalGroundShape); // This method copies both the mesh 'footprint' AND the mesh overrides.
                    // However the mesh assigned to 'originalGroundShape' was a btBoxShape(btVector3(groundHalfDimension,groundHalfDimension,groundHalfDimension)), so:
                    const btVector3 additionalScaling = btVector3(1.0f,0.1f,0.5f);
                    GetMeshOverrides(*groundShape).scaling*= additionalScaling; // original scaling was zero, so we could just use '=' instead of '*='
                    GetMeshOverrides(*groundShape).offset*= additionalScaling;  // this can be a bit more difficult to understand..., but the original offset (due to the fact that a plane mesh was assigned to a box shape)
                                                                                // was calibrated on the original ground shape (vertical half distance to the box center); thus now that distance must be updated.
                }
                //----------------------------------------------------------------------------------------------------
		groundTransform.setOrigin(btVector3(0,5.0f,-groundHalfDimension));
		groundTransform.setRotation(btQuaternion(btVector3(1,0,0),btRadians(30)));
		body = localCreateRigidBody(0.,groundTransform,groundShape);
		body->setFriction(0.5f);
		body->setRestitution(0.5f);
		
		groundTransform.setOrigin(btVector3(0,5.0f,groundHalfDimension));
		groundTransform.setRotation(btQuaternion(btVector3(1,0,0),btRadians(-30)));
		body = localCreateRigidBody(0.,groundTransform,groundShape);
		body->setFriction(0.5f);
		body->setRestitution(0.5f);			
	}
	
	
	
	//=======
	// SLEDGE
	//=======
	#ifdef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		btTriangleMeshEx* pmesh = new btTriangleMeshEx();
		gPersistentMeshes.push_back(pmesh);
		btTriangleMeshEx& mesh = *pmesh;		// Now the mesh is persistent	
	#else //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		btTriangleMeshEx  mesh;	
	#endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO

	char modelFullPath[1025];	// "appHACDDemoMeshes/sledge.obj", but in global path (more robust)
	{
		// This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
		// (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
		modelFullPath[0]='\0';	
		GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,"appHACDDemoMeshes/sledge.obj");
	}	

		
	if (!mesh.loadFromFile(modelFullPath))	{
		printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
		return;
	}
	//else mesh.saveToOFFFile("MySledge.off");
	//mesh.displayDebugInfo();

	//mesh.center();	// This centers the mesh (i.e. will shift the center of mass to the center).
	mesh.setHeight(4);	// in meters! This scales the mesh in an uniform way so that the height is selected. BTW: the scale in all this demo is too big. No time to fix it...

	btCollisionShape* shape = NULL;		
	btScalar mass(2.);	// Kg!
	btTransform T; T.setIdentity(); 
	const btScalar yPos(37.5);
	T.setOrigin(btVector3(-20,yPos,-groundHalfDimension*1.25));	
	T.setRotation(btQuaternion(btVector3(1,0,0),btRadians(30)));

	btHACDCompoundShape::Params params;
	params.keepSubmeshesSeparated = true;	// This mesh (sledge.obj) is composed by two subparts, the lower part (ski) and the upper part (chassis), we keep them separate to apply a different friction for each of them
	// With "params.keepSubmeshesSeparated = true" the following values are relative to EACH subpart of the mesh (not to the whole mesh):
	params.nClusters = 12;		// (1) Min number of resulting sub shapes (but now for each subpart!). We exclude the possibility of a too low number of convex hulls.
								// 4 is good for he lower part (the best for sliding), but will "close" the hole in the upper part (this didn't happen in an older version of the HACD library)
								// 8 is good for the upper part hole, but will cause bad sliding in the lower part.
								// 12 seems good for both, but consumes a lot of resources (and in a previous version 4 worked better than 12 now).							
	// OK, I've added some methods to detach some parameters: the following two lines refer to the first submesh:
	params.keepSubmeshesSeparatedNClusters.push_back(4);	// Trick: this way we explicitely set nClusters for submesh[0]. Missing values or values<=0 are replaced by the 'global' nClusters.
	//params.keepSubmeshesSeparatedMaxHullVertices.push_back(10);	// Trick: this way we explicitely set maxHullVertices for submesh[0]. Missing values or values<0 are replaced by the 'global' maxHullVertices.
	//params.concavity = 200;	// (200) (This controls the number of hulls too) lower values => more shapes (and slower runtime performance)
	params.addExtraDistPoints =  params.addFacesPoints = true;	// This 'should' and 'used to' handle holes in concave meshes better
	//params.displayDebugInfo = true;

	{
		#ifndef USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		shape = new btHACDCompoundShape(mesh,params);	// This might take a while
		btHACDCompoundShape::upcast(shape)->getSubmeshIndexOfChildShapesMapCopy(gSledgeShapeSubmeshIndexOfChildShapes);	// SERIALIZATION PROBLEM
		/*
			We don't save/load the shape back, but in that case there's a problem, since btHACDCompoundShapes are saved as btCompoundShapes, and
			thus we can't load back gSledgeShapeSubmeshIndexOfChildShapes.
			We should hard-code it in that case (when decomposition is performed, the content of that array is always displayed in the console window). 
		*/
		#else 	//USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO
		shape = new btGImpactMeshShape(mesh);	// Please note that "mesh" must be persistent, otherwise a crash occurs when it goes out of scope.
		static_cast < btGImpactMeshShape* > (shape)->updateBound();
		#endif //USE_GIMPACTMESHSHAPE_IN_SLEDGE_DEMO

                // Optional Stuff ('r' key)-----------------------------------------------------------------
                {
                    // The sledge mesh has no textures or materials, but we can set them:
                    mesh.setMeshSubPartMaterial(0,Material(1,1,0));
                    mesh.setMeshSubPartMaterial(1,Material(1,0,0));
                    this->AssignMesh(shape,mesh,true);
                }
                //------------------------------------------------------------------------------------------


		gSledgeShape = shape;	//Global
				
		btRigidBody* body = localCreateRigidBody(mass,T,shape);
		body->setCollisionFlags(body->getCollisionFlags() | btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK);
		/*
		#ifdef USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		body->setAnisotropicFriction(gAnisotropicFrictionOLowerPartOfTheSledge);
		#endif //USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		*/
	}	


	T.setOrigin(btVector3(20,yPos,-groundHalfDimension*1.25));	
	{
		btRigidBody* body = localCreateRigidBody(mass,T,shape);
		body->setCollisionFlags(body->getCollisionFlags() | btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK);
		/*
		#ifdef USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		body->setAnisotropicFriction(gAnisotropicFrictionOLowerPartOfTheSledge);
		#endif //USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		*/
                #ifdef USE_COLLISION_OBJECT_USER_POINTER
                // Optional Stuff: ('r' key and the def above defined)
                {
                    // Please see the TableAndChairs Demo for more about this.
                    this->AssignMeshCopy(body,gSledgeShape);
                    static const Material myStaticMaterial(0,1,0);	// Material overrides must be persistent.
                    btTriangleMeshExFootPrint::Overrides& ovr = GetMeshOverrides(*body);
                    ovr.materialArray.resize(2);
                    ovr.materialArray[0]=NULL;  // We don't override the 1st material
                    ovr.materialArray[1]=&myStaticMaterial;
                }
                #endif // USE_COLLISION_OBJECT_USER_POINTER
	}

	T.setOrigin(btVector3(0,yPos,-groundHalfDimension*1.25));
	T.setRotation(btQuaternion(btVector3(0,0,1),SIMD_PI));	// The sledge at the center will start upside down, so we'll see that the friction will stop it soon.
		
	{
		btRigidBody* body = localCreateRigidBody(mass,T,shape);
		body->setCollisionFlags(body->getCollisionFlags() | btCollisionObject::CF_CUSTOM_MATERIAL_CALLBACK);
		/*
		#ifdef USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		body->setAnisotropicFriction(gAnisotropicFrictionOLowerPartOfTheSledge);
		#endif //USE_ANOSOTROPIC_FRICTION_IN_SLEDGE_DEMO
		*/
                #ifdef USE_COLLISION_OBJECT_USER_POINTER
                // Optional Stuff: ('r' key and the def above defined)
                {
                // Please see the TableAndChairs Demo for more about this.
                this->AssignMeshCopy(body,gSledgeShape);
                static const Material myStaticMaterial(0,0,1);	// Material overrides must be persistent.
                btTriangleMeshExFootPrint::Overrides& ovr = GetMeshOverrides(*body);
                ovr.materialArray.resize(2);
                ovr.materialArray[0]=NULL;  // We don't override the 1st material
                ovr.materialArray[1]=&myStaticMaterial;
                }
                #endif // USE_COLLISION_OBJECT_USER_POINTER
	}	
			
	printf("'SlidingSledgeDemo' ready.\n");
	printf("Press '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");

}
#ifndef EXCLUDE_VHACD
// VHACD test
void	HACDDemo::initPhysicsForDemoVHACDTest()	{
    // This demo is a quick and dirty "BunnyFamilyDemo" replacement, just to test the usage of the btVHACDCompoundShape class.

    // OVERVIEW:
    // 1) VHACD can be used for MANIFOLD input meshes ONLY (i.e. fully closed meshes)
    // 2) It produces NON-OVERLAPPING child shapes (good for splitting them into the world)

    // Quick note: at the moment VHACD does not support setting many values, such as params.nClusters params.maxHullVertices (thus the child shapes
    // "look better", because are made of a lot of vertices, which is probably not optimal for performance)

        printf("\nStarting 'VHACDTestDemo'. Please wait...\n");

        createGround();

        btTriangleMeshEx mesh;			// Our mesh class. Compound shapes don't require it to be persistent (so we scope it).
        std::string meshNames[]={"appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultBunnyMesh.off",
                                 "appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultTorusMesh.off",
                                 "appHACDDemoMeshes/table.obj",
                                 "appHACDDemoMeshes/chair.obj",
                                 "appHACDDemoMeshes/sledge.obj"
                                };
        const int numMeshNames = sizeof(meshNames)/sizeof(meshNames[0]);
        static int staticSelectedMesh = -1;++staticSelectedMesh;
        int selectedMesh = staticSelectedMesh%numMeshNames;

        {
            if (selectedMesh<0 || selectedMesh>=numMeshNames) selectedMesh = 0;

            char modelFullPath[1025];	// "appHACDDemoMeshes/MeshesFromAppGImpactTestDemo/BulletDefaultBunnyMesh.off"
            // This is just because when executing appHACDDemo.exe from a different directory, the current directory is not the appHACDDemo.exe directory,
            // and I don't want to use a windows-specific method to set it (::SetCurrentDirectory(...)). This demo is intended to work even in cmdline mode
            // (with drag and drop on the .exe file): this makes the program start from a different current directory. You don't have to do this in your programs!
            modelFullPath[0]='\0';
            GetExeFileDirectoryPlusRelativeFilePath(modelFullPath,1024,gExeFileFullPath,meshNames[selectedMesh].c_str());
            if (!mesh.loadFromFile(modelFullPath))	{
                printf("ERROR: Can't load file \"%s\". Please copy the folder \"appHACDDemoMeshes\" in the same directory of addHACDDemo and retry.\n",modelFullPath);
                return;
            }
        }

        //mesh.center();	// This centers the mesh (i.e. will shift the center of mass to the center).
        if (selectedMesh==1) mesh.transform(btTransform(btQuaternion(btVector3(1,0,0),SIMD_HALF_PI),btVector3(0,0,0)));	//We rotate the mesh 90 degree around the X axis
        else if (selectedMesh==2) mesh.transform(btTransform(btQuaternion(btVector3(0,1,0),SIMD_HALF_PI),btVector3(0,0,0)));	//We rotate the mesh 90 degree around the Y axis
        mesh.setHeight(8);	// in meters! This scales the mesh in an uniform way so that the height is selected.

        btVHACDCompoundShape::Params params;
        if (selectedMesh<3) params.resetForSpeed();
        else params.resetForCommonUsage();
        params.shrinkObjectInwardsToCompensateCollisionMargin = true;   // Without it some gaps between child shapes can be visible in some cases
        params.convexHullsCollisionMargin = 0.02;
        //------- These 2 can be helpful only when we want to split the child shapes as individual rigid bodies------------
        params.useStanMelaxVolumeIntegrationForClusterOrigins = true;   // (optional) puts each cluster's origin in its "uniform mass based mesh center" instead of in its AABB center.
        params.storeAdditionalInfoOfChildShapes = true;                 // This takes a bit of time, but can be used to calculate each cluster's mass better
        //-----------------------------------------------------------------------------------------------------------------
        btVHACDCompoundShape* shape = new btVHACDCompoundShape(mesh,params);	// This might take a while

        if (selectedMesh<2) mesh.setMeshSubPartMaterial(0,gPersistentMaterials[0]);	// The bunny mesh will be white without this line... (actually materials don't have to be persistent here...)
        this->AssignMesh(shape,mesh,selectedMesh>1);	// New: Optional Stuff ('r' key') -- creates a series of display lists, so that the mesh can go safely out of scope later, and stuffs it into the shape user pointer

        btScalar mass(1.);
        btTransform T; T.setIdentity(); T.setOrigin(btVector3(-10,10,10));

        // create rigid body and add it to the world (ALL replaceble by: localCreateRigidBody(mass,T,shape);)
        {
                //rigidbody is dynamic if and only if mass is non zero, otherwise static
                bool isDynamic = (mass != 0.f);
                btVector3 localInertia(0,0,0);
                if (isDynamic) shape->calculateLocalInertia(mass,localInertia);
                //using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
                btDefaultMotionState* myMotionState = new btDefaultMotionState(T);
                btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,shape,localInertia);
                btRigidBody* body = new btRigidBody(rbInfo);

                //add the body to the dynamics world
                m_dynamicsWorld->addRigidBody(body);
        }


        // OK. So far, so good. Now we add a row with scaled shapes:

            T.setIdentity(); T.setOrigin(btVector3(-50,10,-20));

            for (int i = 1;i<5;i++)	{
                const btScalar uniformScalingFactor = 0.5 * i;
                btCompoundShape* cs = btUtils::GetUniformScalingCompoundShape(shape,uniformScalingFactor);

                const btScalar mass(uniformScalingFactor*3);
                T.getOrigin().setX(T.getOrigin().x()+20);

                // Optional Stuff - 'r' key
                {
                    //AssignMesh(cs,mesh,false,btVector3(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor));	// Deprecated: this creates a brand new series of display lists for the same mesh ('r' key') (bool arg = useFlatNormals)
                    AssignMeshCopy(cs,shape);	// Cheaper: the display lists are shared, but we must still override the mesh scaling
                    btTriangleMeshExFootPrint::Overrides& ovrs = GetMeshOverrides(*cs);
                    ovrs.scaling = btVector3(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor);	// needed to override the mesh scaling
                    if (selectedMesh<2) ovrs.materialArray.push_back(&gPersistentMaterials[i%gNumPersistentMaterials]);	//color override: 1st material (and unique for this mesh)
                    // VERY IMPORTANT: When we override a material inside any 'Mesh Overrides' it MUST be persistent (that's the only important thing to remember)!!!
                }


                if (i<4)	/*btRigidBody* body = */localCreateRigidBody(mass,T,cs);	// Equivalent to the code above (INCLUDING addRigidBody())
                else	{
                    // We add a center of mass (COM) offset to the last body:
                    const btVector3 curBodyHalfExtents = btUtils::GetAabbHalfExtents(shape)*uniformScalingFactor;
                    const btVector3 comOffset(0,-curBodyHalfExtents.y()*btScalar(3.0/4.0),-curBodyHalfExtents.z()*btScalar(2.0/3.0));
                    btRigidBody* body = localCreateRigidBodyWithCOMOffset(mass,T,cs,comOffset);	// Equivalent to the code above (INCLUDING addRigidBody())

                    // Debug Stuff (please check if it's correct)
                    /*
                    {
                        btDefaultMotionState* ms = (btDefaultMotionState*) body->getMotionState();
                        if (ms)	{
                            btTransform T;
                            ms->getWorldTransform(T);
                            printf("World Position Of COM:		(%1.2f,%1.2f,%1.2f)\n",T.getOrigin().x(),T.getOrigin().y(),T.getOrigin().z());
                        }
                    }
                    #ifdef USE_COLLISION_OBJECT_USER_POINTER 	// defined in "GlutDemoApplicationEx.h": allows people to retrieve the transform of the body "outside" the center of mass.
                                                                // when defined btCollisionObject::get/setUserPointer() cannot be used anymore, but can be replaced by this->Get/SetUserPointer(...)
                    {
                        btMyDefaultMotionState* ms = btMyDefaultMotionState::upcast(body);
                        if (ms)	{
                            btTransform T;
                            ms->getWorldTransformOutsideCOM(T);
                            printf("World Position Outside COM:	(%1.2f,%1.2f,%1.2f)\n",T.getOrigin().x(),T.getOrigin().y(),T.getOrigin().z());
                        }
                    }
                    #endif //USE_COLLISION_OBJECT_USER_POINTER
                    */
                }
            }

            //New: We add a copy of the original-sized body, but we split it into its child shapes and create a new body for each---------------------
            {
                T.setOrigin(btVector3(10,10,10));
                int sz = shape->getNumChildShapes();
                btScalar clusterMass = mass;
                if (sz>0) clusterMass/=(btScalar)sz;    //There are advanced ways of calculating it when "params.storeAdditionalInfoOfChildShapes==true" (see inside loop)
                Material& clusterMaterial = gPersistentMaterials[6%gNumPersistentMaterials];
                for (int i=0;i<sz;i++)   {
                    if (params.storeAdditionalInfoOfChildShapes)    {
                        const btVHACDCompoundShape::AdditionalInfoOfChildShapes& ais = shape->getAdditionalInfoOfChildShapes();
                        const btVHACDCompoundShape::AdditionalInfoOfChildShape& ai = ais[i];
                        clusterMass = ai.totalVolumeFraction*mass; // Actually I'm not 100% sure that the members of these structures are correct (!)
                        btRigidBody* body = localCreateRigidBody(clusterMass,T*shape->getChildTransform(i),shape->getChildShape(i));
                        // Optional Stuff ('r' key):
                        {
                            btTriangleMeshEx::btMeshSubPart meshSubPart;
                            meshSubPart.verts = ai.vertsConcave;   // VHACD gives us verts and inds of the CONCAVE shards (HACD instead convex verts and inds AFAIR (TODO: check)),
                            meshSubPart.inds = ai.indsConcave;     // but not the texture coordinates... so we cut them.
                            //(ai.verts and ai.inds refer to the convex hull data instead, that are used in all the calculations inside <btVHACDCompoundShape.h>)
                            meshSubPart.material = clusterMaterial; //This time it can be non-persistent AFAIR
                            btTriangleMeshEx clusterMesh;
                            clusterMesh.addMeshSubPart(meshSubPart);
                            this->AssignMesh(body,clusterMesh,selectedMesh>1);
                        }
                    }
                }

            }
            //----------------------------------------------------------------------------------------------------------------------------------------

    printf("'VHACDTestDemo' ready.\n");
    printf("Press 'space' to restart this demo with a different mesh.\nPress '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");
}
#endif //EXCLUDE_VHACD


// A bit difficult (and uses STL)
void	HACDDemo::initPhysicForCommandlineUsage()	{	
        // A few (non-persistent) material definition (for later use)------------------
        Material materials[4];	// we will use mat[0] to display the mesh, in case it has no submeshes and no textures and the unique material is the default one (white), because the default white material sucks...
                                // The others are only used when displaying Bullet Collision Shapes With Mesh Interfaces inside them composed of more than one subparts.
        const int numMaterials = sizeof(materials)/sizeof(materials[0]);
        {
            Material& mat(materials[0]);
            mat.amb[0] = 0.f;	mat.amb[1] = 0.5f;	mat.amb[2] = 0.5f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.f;	mat.dif[1] = 0.f;	mat.dif[2] = 1.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[1]);
            mat.amb[0] = 0.3f;	mat.amb[1] = 0.25f;	mat.amb[2] = 0.0f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.6f;	mat.dif[1] = 0.f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[2]);
            mat.amb[0] = 0.3f;	mat.amb[1] = 0.25f;	mat.amb[2] = 0.1f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.6f;	mat.dif[1] = 0.6f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[3]);
            mat.amb[0] = 0.f;	mat.amb[1] = 0.3f;	mat.amb[2] = 0.25f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.f;	mat.dif[1] = 0.6f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        // Variables:-----------------------------------------------------------------
         //-------------------------------
	btHACDCompoundShape::Params params;
	bool extraParamUseAutoGeneratedSaveNameToSaveShape;
	bool extraParamInvertYZ;
	bool extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape;
	bool extraParamIgnoreAllParamsAndCreateGImpaceMeshShape;
	bool extraParamDecomposeBCSInputFileIfShapeIsCompatible;
    bool extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape;
    bool extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable;
	bool cmdlineParamsOk(false);	
	bool cmdlineParamsLoaded(false);
	btTriangleMeshEx* pmesh = new btTriangleMeshEx();
	gPersistentMeshes.push_back(pmesh);
	btTriangleMeshEx& mesh = *pmesh;	// Persistent mesh	
	char cmdLineParamsFullPath[1025];	
	{
	cmdLineParamsFullPath[0]='\0';
	strcpy(cmdLineParamsFullPath,"appHACDDemo.exe.cmdlineParams.txt");
	// This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
	// and I don't want to use a windows-specific method to set it...
	GetExeFileDirectoryPlusRelativeFilePath(cmdLineParamsFullPath,1024,gExeFileFullPath,cmdLineParamsFullPath);			
	}
        bool meshLoadedFromStridingMeshInterface(false);
	//----------------------------------------------------------------------------
	
	const bool hasBulletExtension = FileHasExtension(gFileFromCommandLine,".bullet");
	if (FileHasExtension(gFileFromCommandLine,".bcs") || hasBulletExtension)	{
                #ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
		if (!cmdlineParamsLoaded) {
			// Needed for 'extraParamDecomposeBCSInputFileIfShapeIsCompatible'
            cmdlineParamsOk = LoadHACDCompoundShapeParams(cmdLineParamsFullPath,params,&extraParamUseAutoGeneratedSaveNameToSaveShape,&extraParamInvertYZ,&extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,&extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,&extraParamDecomposeBCSInputFileIfShapeIsCompatible,&extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape,&extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable);
			cmdlineParamsLoaded = true;
            #ifndef EXCLUDE_VHACD
            if (extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable) {
                initPhysicForCommandlineUsageVHACD();
                return;
            }
            #endif //EXCLUDE_VHACD
		}		
		#endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
		
		printf("Loading Bullet Collision Shape from cmdline: \"%s\"\n",gFileFromCommandLine);
		if (hasBulletExtension) printf("Warning: In files with the \".bullet\" extension, only the first collision shape is loaded.\n");
		btCollisionShape* shape = btUtils::Load(gFileFromCommandLine);
		if (!shape) {
			printf("Error: File not found or no collision shape found inside: \"%s\"\n",gFileFromCommandLine);
			return;
		}	
		#ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
                if (!extraParamDecomposeBCSInputFileIfShapeIsCompatible || !(meshLoadedFromStridingMeshInterface=mesh.loadFromMeshInterfaceInsideBulletCollisionShape(shape)))
		#endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
		{
			//Just display shape:
			btScalar mass = 1.;		
			btScalar yPos = btUtils::GetAabbFullExtents(shape).y();
			btTransform T; T.setIdentity(); T.setOrigin(btVector3(0,yPos,0));	
			switch (shape->getShapeType())	{
			// Convex shapes:
			//case TRIANGLE_SHAPE_PROXYTYPE:
			//case CONVEX_TRIANGLEMESH_SHAPE_PROXYTYPE:
			// Concave shapes:
			case TRIANGLE_MESH_SHAPE_PROXYTYPE:
			case SCALED_TRIANGLE_MESH_SHAPE_PROXYTYPE:
			case FAST_CONCAVE_MESH_PROXYTYPE:
			case TERRAIN_SHAPE_PROXYTYPE:
			case MULTIMATERIAL_TRIANGLE_MESH_PROXYTYPE:	
			case STATIC_PLANE_PROXYTYPE:
			case CUSTOM_CONCAVE_SHAPE_TYPE:				
					mass = btScalar(0.);
					break;
			}
			localCreateRigidBody(mass,T,shape);	
		
			if (shape)	{
				if (shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE) {
					printf("NumChildShapes:		%1d\n",static_cast <btCompoundShape*>(shape)->getNumChildShapes());
					DisplayAvgNumberOfHullVerticesInChildShapes(static_cast <btCompoundShape*>(shape));
				}	
				const btVector3 fullExtents = btUtils::GetAabbFullExtents(shape);
				printf("FullAabbExtents:	(%1.4f,%1.4f,%1.4f)\n",fullExtents.x(),fullExtents.y(),fullExtents.z());
                                setCameraDistance(btScalar(3.)*fullExtents.z());
                                createGround(fullExtents.z()*btScalar(0.5)>100 ?  fullExtents.z()*btScalar(0.5) : 100);

                                if (!meshLoadedFromStridingMeshInterface) meshLoadedFromStridingMeshInterface=mesh.loadFromMeshInterfaceInsideBulletCollisionShape(shape);
                                if (meshLoadedFromStridingMeshInterface) {
                                    for (int i=0;i<mesh.getNumMeshSubParts();i++) mesh.setMeshSubPartMaterial(i,materials[i%numMaterials]);
                                    AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
                                }
			}
                        createGround();

			printf("\nPress '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");		
			return;
			}
	}
	
	printf("Processing mesh from cmdline: \"%s\"\n\n",gFileFromCommandLine);	
	printf("-----------------------------------------------------\n");
   if (!cmdlineParamsLoaded) {
        cmdlineParamsOk = LoadHACDCompoundShapeParams(cmdLineParamsFullPath,params,&extraParamUseAutoGeneratedSaveNameToSaveShape,&extraParamInvertYZ,&extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,&extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,&extraParamDecomposeBCSInputFileIfShapeIsCompatible,&extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape,&extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable);
		cmdlineParamsLoaded = true;
        #ifndef EXCLUDE_VHACD
        if (extraParamIgnoreAllParamsAndUseVHACDInsteadIfAvailable) {
            initPhysicForCommandlineUsageVHACD();
            return;
        }
        #endif //EXCLUDE_VHACD
	}
	if (!cmdlineParamsOk) printf("File \"%s\" not found, using defaults.\n",cmdLineParamsFullPath);
	else printf("File \"%s\" loaded.\n",cmdLineParamsFullPath);	
    if (extraParamUseAutoGeneratedSaveNameToSaveShape && params.optionalBCSSaveFilePath.size()>0) {
		params.optionalBCSSaveFilePath = gFileFromCommandLine;
		params.optionalBCSSaveFilePath+=".bcs";
	}
    if (!extraParamUseAutoGeneratedSaveNameToSaveShape && params.optionalBCSSaveFilePath.size()>0)	{
		// This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it...
		char temp[1025];
		temp[0]='\0';
		GetExeFileDirectoryPlusRelativeFilePath(temp,1024,gExeFileFullPath,params.optionalBCSSaveFilePath.c_str());
		params.optionalBCSSaveFilePath = temp;
    }
    if (params.optionalVRMLSaveFilePath.size()>0)	{
		// This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
		// and I don't want to use a windows-specific method to set it...
		char temp[1025];
		temp[0]='\0';
		GetExeFileDirectoryPlusRelativeFilePath(temp,1024,gExeFileFullPath,params.optionalVRMLSaveFilePath.c_str());
		params.optionalVRMLSaveFilePath = temp;		
	}
    if (cmdlineParamsOk) {
		params.display(true);
		if (extraParamUseAutoGeneratedSaveNameToSaveShape) 			printf("extraParams.useAutoGeneratedSaveNameToSaveShape		= true;\n");
		if (extraParamInvertYZ) 									printf("extraParams.invertYZ								= true;\n");
		if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape) printf("extraParams.ignoreAllParamsAndCreateBvhTriangleMeshShape= true;\n");
		if (extraParamIgnoreAllParamsAndCreateGImpaceMeshShape) 	printf("extraParams.ignoreAllParamsAndCreateGImpaceMeshShape= true;\n");		
		if (extraParamDecomposeBCSInputFileIfShapeIsCompatible)		printf("extraParams.decomposeBCSInputFileIfShapeIsCompatible= true;\n");
        if (!extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape) printf("extraParams.useFlatNormalsWhenAssigningAMeshToTheDecomposedShape= false;\n");
    }
	printf("-----------------------------------------------------\n");
	
	
	printf("Loading mesh from commandline: \"%s\"\n",gFileFromCommandLine);
	bool ok = mesh.getNumMeshSubParts()>0;	// this is because we may have already loaded it 
	if (!ok) ok = mesh.loadFromFile(gFileFromCommandLine,extraParamInvertYZ);
	if (!ok) {
		printf("ERROR: File not found or unsupported model format or corrupted file.\n");
		return;
	}	
	printf("Model loaded.\n");
	if (params.displayDebugInfo)	{
		printf("Model info: -----------------------------------------\n");
		mesh.displayDebugInfo();
		printf("-----------------------------------------------------\n");
	}	
	
	btCollisionShape* shape = NULL;		
	btScalar mass = 1.;
	
			
	if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape || extraParamIgnoreAllParamsAndCreateGImpaceMeshShape)	{
		// Actually not all params are ignored...
		// params.decomposeAScaledCopyOfTheMesh
		// params.decomposeACenteredCopyOfTheMesh
		// params.convexHullsCollisionMargin (used for the whole shape)
		// params.optionalBCSSaveFilePath ( if "" the mesh will not be saved)
		if (params.decomposeACenteredCopyOfTheMesh) mesh.center();
		if (params.decomposeATranslatedCopyOfTheMesh!=btVector3(0,0,0)) mesh.shift(params.decomposeATranslatedCopyOfTheMesh);		
		if (params.decomposeAScaledCopyOfTheMesh!=btVector3(1,1,1)) mesh.scale(params.decomposeAScaledCopyOfTheMesh);

		if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape)	{
			const bool useQuantizedAabbCompression(true);
			shape = new btBvhTriangleMeshShape(mesh,useQuantizedAabbCompression);
			shape->setMargin(params.convexHullsCollisionMargin); 
			mass = 0.;		
		}
		else if (extraParamIgnoreAllParamsAndCreateGImpaceMeshShape)	{
			btGImpactMeshShape* gishape = new btGImpactMeshShape(mesh);
			gishape->setLocalScaling(btVector3(1.f,1.f,1.f));
			gishape->setMargin(params.convexHullsCollisionMargin); 
			gishape->updateBound();
			shape = gishape;
		}
		if (shape && params.optionalBCSSaveFilePath.size()>0) {
			if (btUtils::Save(shape,params.optionalBCSSaveFilePath.c_str())) printf("Bullet collision shape saved to file: %s\n",params.optionalBCSSaveFilePath.c_str());
		}	
	}
	else {
		printf("Performing decomposition...\n");
		shape = new btHACDCompoundShape(mesh,params);
		printf("Decomposition performed.\n");			

		// Important: we modify the mesh AFTER HACD shape creation, so that we can assign 'shape' (later) to a new mesh that matches the decomposed copy of the mesh
		if (params.decomposeACenteredCopyOfTheMesh) mesh.center();
		if (params.decomposeATranslatedCopyOfTheMesh!=btVector3(0,0,0)) mesh.shift(params.decomposeATranslatedCopyOfTheMesh);		
		if (params.decomposeAScaledCopyOfTheMesh!=btVector3(1,1,1)) mesh.scale(params.decomposeAScaledCopyOfTheMesh);		
		// Please note that the params above aren't very useful when <btHACDCompoundShape.h> is used at runtime: it's better to center, translate or scale the mesh (btTriangleMeshEx)
		// directly, before decomposition, and leave the params above in their default state. This way there's a 1:1 match between the shape and the mesh.
	}		
	
	if (shape) {		
		if (shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE) {
			printf("NumChildShapes:		%1d\n",static_cast <btCompoundShape*>(shape)->getNumChildShapes());
			DisplayAvgNumberOfHullVerticesInChildShapes(static_cast <btCompoundShape*>(shape));
		}
		const btVector3 fullExtents = btUtils::GetAabbFullExtents(shape);
		printf("FullAabbExtents:	(%1.4f,%1.4f,%1.4f)\n",fullExtents.x(),fullExtents.y(),fullExtents.z());
                setCameraDistance(btScalar(3.)*fullExtents.z());
		
		btScalar yPos = fullExtents.y();
		if (params.decomposeACenteredCopyOfTheMesh) yPos*=0.5f;
		btTransform T; T.setIdentity(); T.setOrigin(btVector3(0,yPos,0));	
		if (shape) localCreateRigidBody(mass,T,shape);	
		
		// Adjust the camera target:
		m_cameraTargetPosition[0] = 0;
		m_cameraTargetPosition[1] = fullExtents.y()*btScalar(0.5); 
		m_cameraTargetPosition[2] = 0;
	}
	
	if (shape) createGround(btUtils::GetAabbHalfExtents(shape).z()>100 ?  btUtils::GetAabbHalfExtents(shape).z() : 100);
	else createGround();
	

	
	// Assign the mesh to the shape, so that it is displayed in one of the rendering modes ('r' key)
        if (meshLoadedFromStridingMeshInterface) {
            for (int i=0;i<mesh.getNumMeshSubParts();i++) mesh.setMeshSubPartMaterial(i,materials[i%numMaterials]);
            this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
        }
        else    {
            bool useMaterialOverride = mesh.getNumMeshSubParts()==1 && mesh.getMeshSubPart(0)->textureId==0;
            if (useMaterialOverride)	{
		const Material& mat = *(mesh.getMeshSubPartMaterial(0));
		if (!(mat.dif[0]==mat.dif[1] && mat.dif[1]==mat.dif[2]))  useMaterialOverride = false;		
            }
            if (!useMaterialOverride) this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
            else {
                this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape,btVector3(1,1,1),materials[0]); // If we don't want flat normals, we can use 'false' here
            }
        }
		
	printf("\nPress '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");		

}

#ifndef EXCLUDE_VHACD   //Tweakable but not here: in HACDDemo.h or in the 'compiler definitions' of the project
void	HACDDemo::initPhysicForCommandlineUsageVHACD() {
    // 95% copied and pasted from the method above... bad coding... but I want to be able to split this part more easily...
        // A few (non-persistent) material definition (for later use)------------------
        Material materials[4];	// we will use mat[0] to display the mesh, in case it has no submeshes and no textures and the unique material is the default one (white), because the default white material sucks...
                                // The others are only used when displaying Bullet Collision Shapes With Mesh Interfaces inside them composed of more than one subparts.
        const int numMaterials = sizeof(materials)/sizeof(materials[0]);
        {
            Material& mat(materials[0]);
            mat.amb[0] = 0.f;	mat.amb[1] = 0.5f;	mat.amb[2] = 0.5f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.f;	mat.dif[1] = 0.f;	mat.dif[2] = 1.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[1]);
            mat.amb[0] = 0.3f;	mat.amb[1] = 0.25f;	mat.amb[2] = 0.0f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.6f;	mat.dif[1] = 0.f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[2]);
            mat.amb[0] = 0.3f;	mat.amb[1] = 0.25f;	mat.amb[2] = 0.1f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.6f;	mat.dif[1] = 0.6f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        {
            Material& mat(materials[3]);
            mat.amb[0] = 0.f;	mat.amb[1] = 0.3f;	mat.amb[2] = 0.25f;	mat.amb[3] = 1.f;
            mat.dif[0] = 0.f;	mat.dif[1] = 0.6f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
            mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.8f;	mat.spe[3] = 1.f;
            mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
            mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
        }
        // Variables:-----------------------------------------------------------------
        //-------------------------------
    btVHACDCompoundShape::Params params;
    bool extraParamUseAutoGeneratedSaveNameToSaveShape;
    bool extraParamInvertYZ;
    bool extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape;
    bool extraParamIgnoreAllParamsAndCreateGImpaceMeshShape;
    bool extraParamDecomposeBCSInputFileIfShapeIsCompatible;
    bool extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape;
    bool cmdlineParamsOk(false);
    bool cmdlineParamsLoaded(false);
    btTriangleMeshEx* pmesh = new btTriangleMeshEx();
    gPersistentMeshes.push_back(pmesh);
    btTriangleMeshEx& mesh = *pmesh;	// Persistent mesh
    char cmdLineParamsFullPath[1025];
    {
    cmdLineParamsFullPath[0]='\0';
    strcpy(cmdLineParamsFullPath,"appHACDDemo.exe.cmdlineParamsVHACD.txt");
    // This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
    // and I don't want to use a windows-specific method to set it...
    GetExeFileDirectoryPlusRelativeFilePath(cmdLineParamsFullPath,1024,gExeFileFullPath,cmdLineParamsFullPath);
    }
        bool meshLoadedFromStridingMeshInterface(false);
    //----------------------------------------------------------------------------

    const bool hasBulletExtension = FileHasExtension(gFileFromCommandLine,".bullet");
    if (FileHasExtension(gFileFromCommandLine,".bcs") || hasBulletExtension)	{
                #ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
        if (!cmdlineParamsLoaded) {
            // Needed for 'extraParamDecomposeBCSInputFileIfShapeIsCompatible'
            cmdlineParamsOk = LoadVHACDCompoundShapeParams(cmdLineParamsFullPath,params,&extraParamUseAutoGeneratedSaveNameToSaveShape,&extraParamInvertYZ,&extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,&extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,&extraParamDecomposeBCSInputFileIfShapeIsCompatible,&extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
            cmdlineParamsLoaded = true;
        }
        #endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE

        printf("Loading Bullet Collision Shape from cmdline: \"%s\"\n",gFileFromCommandLine);
        if (hasBulletExtension) printf("Warning: In files with the \".bullet\" extension, only the first collision shape is loaded.\n");
        btCollisionShape* shape = btUtils::Load(gFileFromCommandLine);
        if (!shape) {
            printf("Error: File not found or no collision shape found inside: \"%s\"\n",gFileFromCommandLine);
            return;
        }
        #ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
                if (!extraParamDecomposeBCSInputFileIfShapeIsCompatible || !(meshLoadedFromStridingMeshInterface=mesh.loadFromMeshInterfaceInsideBulletCollisionShape(shape)))
        #endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
        {
            //Just display shape:
            btScalar mass = 1.;
            btScalar yPos = btUtils::GetAabbFullExtents(shape).y();
            btTransform T; T.setIdentity(); T.setOrigin(btVector3(0,yPos,0));
            switch (shape->getShapeType())	{
            // Convex shapes:
            //case TRIANGLE_SHAPE_PROXYTYPE:
            //case CONVEX_TRIANGLEMESH_SHAPE_PROXYTYPE:
            // Concave shapes:
            case TRIANGLE_MESH_SHAPE_PROXYTYPE:
            case SCALED_TRIANGLE_MESH_SHAPE_PROXYTYPE:
            case FAST_CONCAVE_MESH_PROXYTYPE:
            case TERRAIN_SHAPE_PROXYTYPE:
            case MULTIMATERIAL_TRIANGLE_MESH_PROXYTYPE:
            case STATIC_PLANE_PROXYTYPE:
            case CUSTOM_CONCAVE_SHAPE_TYPE:
                    mass = btScalar(0.);
                    break;
            }
            localCreateRigidBody(mass,T,shape);

            if (shape)	{
                if (shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE) {
                    printf("NumChildShapes:		%1d\n",static_cast <btCompoundShape*>(shape)->getNumChildShapes());
                    DisplayAvgNumberOfHullVerticesInChildShapes(static_cast <btCompoundShape*>(shape));
                }
                const btVector3 fullExtents = btUtils::GetAabbFullExtents(shape);
                printf("FullAabbExtents:	(%1.4f,%1.4f,%1.4f)\n",fullExtents.x(),fullExtents.y(),fullExtents.z());
                                setCameraDistance(btScalar(3.)*fullExtents.z());
                                createGround(fullExtents.z()*btScalar(0.5)>100 ?  fullExtents.z()*btScalar(0.5) : 100);

                                if (!meshLoadedFromStridingMeshInterface) meshLoadedFromStridingMeshInterface=mesh.loadFromMeshInterfaceInsideBulletCollisionShape(shape);
                                if (meshLoadedFromStridingMeshInterface) {
                                    for (int i=0;i<mesh.getNumMeshSubParts();i++) mesh.setMeshSubPartMaterial(i,materials[i%numMaterials]);
                                    AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
                                }
            }
                        createGround();

            printf("\nPress '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");
            return;
            }
    }

    //printf("Processing mesh from cmdline: \"%s\"\n\n",gFileFromCommandLine);
    //printf("-----------------------------------------------------\n");
    if (!cmdlineParamsLoaded) {
        cmdlineParamsOk = LoadVHACDCompoundShapeParams(cmdLineParamsFullPath,params,&extraParamUseAutoGeneratedSaveNameToSaveShape,&extraParamInvertYZ,&extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape,&extraParamIgnoreAllParamsAndCreateGImpaceMeshShape,&extraParamDecomposeBCSInputFileIfShapeIsCompatible,&extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
        cmdlineParamsLoaded = true;
    }
    if (!cmdlineParamsOk) printf("File \"%s\" not found, using defaults.\n",cmdLineParamsFullPath);
    else printf("File \"%s\" loaded.\n",cmdLineParamsFullPath);
    if (extraParamUseAutoGeneratedSaveNameToSaveShape && params.optionalBCSSaveFilePath.size()>0) {
        params.optionalBCSSaveFilePath = gFileFromCommandLine;
        params.optionalBCSSaveFilePath+=".bcs";
    }
    if (!extraParamUseAutoGeneratedSaveNameToSaveShape && params.optionalBCSSaveFilePath.size()>0)	{
        // This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
        // and I don't want to use a windows-specific method to set it...
        char temp[1025];
        temp[0]='\0';
        GetExeFileDirectoryPlusRelativeFilePath(temp,1024,gExeFileFullPath,params.optionalBCSSaveFilePath.c_str());
        params.optionalBCSSaveFilePath = temp;
    }
    /*if (params.optionalVRMLSaveFilePath.size()>0)	{
        // This is because when dragging a model from another folder the current directory is not the appHACDDemo.exe directory,
        // and I don't want to use a windows-specific method to set it...
        char temp[1025];
        temp[0]='\0';
        GetExeFileDirectoryPlusRelativeFilePath(temp,1024,gExeFileFullPath,params.optionalVRMLSaveFilePath.c_str());
        params.optionalVRMLSaveFilePath = temp;
    }*/
    if (cmdlineParamsOk) {
        params.display(true);
        if (extraParamUseAutoGeneratedSaveNameToSaveShape) 			printf("extraParams.useAutoGeneratedSaveNameToSaveShape		= true;\n");
        if (extraParamInvertYZ) 									printf("extraParams.invertYZ								= true;\n");
        if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape) printf("extraParams.ignoreAllParamsAndCreateBvhTriangleMeshShape= true;\n");
        if (extraParamIgnoreAllParamsAndCreateGImpaceMeshShape) 	printf("extraParams.ignoreAllParamsAndCreateGImpaceMeshShape= true;\n");
        if (extraParamDecomposeBCSInputFileIfShapeIsCompatible)		printf("extraParams.decomposeBCSInputFileIfShapeIsCompatible= true;\n");
        if (!extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape) printf("extraParams.useFlatNormalsWhenAssigningAMeshToTheDecomposedShape= false;\n");
    }
    printf("-----------------------------------------------------\n");


    printf("Loading mesh from commandline: \"%s\"\n",gFileFromCommandLine);
    bool ok = mesh.getNumMeshSubParts()>0;	// this is because we may have already loaded it
    if (!ok) ok = mesh.loadFromFile(gFileFromCommandLine,extraParamInvertYZ);
    if (!ok) {
        printf("ERROR: File not found or unsupported model format or corrupted file.\n");
        return;
    }
    printf("Model loaded.\n");
    if (params.displayDebugInfo)	{
        printf("Model info: -----------------------------------------\n");
        mesh.displayDebugInfo();
        printf("-----------------------------------------------------\n");
    }    

    btCollisionShape* shape = NULL;
    btScalar mass = 1.;


    if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape || extraParamIgnoreAllParamsAndCreateGImpaceMeshShape)	{
        // Actually not all params are ignored...
        // params.decomposeAScaledCopyOfTheMesh
        // params.decomposeACenteredCopyOfTheMesh
        // params.convexHullsCollisionMargin (used for the whole shape)
        // params.optionalBCSSaveFilePath ( if "" the mesh will not be saved)
        if (params.decomposeACenteredCopyOfTheMesh) mesh.center();
        if (params.decomposeATranslatedCopyOfTheMesh!=btVector3(0,0,0)) mesh.shift(params.decomposeATranslatedCopyOfTheMesh);
        if (params.decomposeAScaledCopyOfTheMesh!=btVector3(1,1,1)) mesh.scale(params.decomposeAScaledCopyOfTheMesh);

        if (extraParamIgnoreAllParamsAndCreateBvhTriangleMeshShape)	{
            const bool useQuantizedAabbCompression(true);
            shape = new btBvhTriangleMeshShape(mesh,useQuantizedAabbCompression);
            shape->setMargin(params.convexHullsCollisionMargin);
            mass = 0.;
        }
        else if (extraParamIgnoreAllParamsAndCreateGImpaceMeshShape)	{
            btGImpactMeshShape* gishape = new btGImpactMeshShape(mesh);
            gishape->setLocalScaling(btVector3(1.f,1.f,1.f));
            gishape->setMargin(params.convexHullsCollisionMargin);
            gishape->updateBound();
            shape = gishape;
        }
        if (shape && params.optionalBCSSaveFilePath.size()>0) {
            if (btUtils::Save(shape,params.optionalBCSSaveFilePath.c_str())) printf("Bullet collision shape saved to file: %s\n",params.optionalBCSSaveFilePath.c_str());
        }
    }
    else {
        printf("Performing decomposition...\n");
        shape = new btVHACDCompoundShape(mesh,params);
        printf("Decomposition performed.\n");

        // Important: we modify the mesh AFTER HACD shape creation, so that we can assign 'shape' (later) to a new mesh that matches the decomposed copy of the mesh
        if (params.decomposeACenteredCopyOfTheMesh) mesh.center();
        if (params.decomposeATranslatedCopyOfTheMesh!=btVector3(0,0,0)) mesh.shift(params.decomposeATranslatedCopyOfTheMesh);
        if (params.decomposeAScaledCopyOfTheMesh!=btVector3(1,1,1)) mesh.scale(params.decomposeAScaledCopyOfTheMesh);
        // Please note that the params above aren't very useful when <btHACDCompoundShape.h> is used at runtime: it's better to center, translate or scale the mesh (btTriangleMeshEx)
        // directly, before decomposition, and leave the params above in their default state. This way there's a 1:1 match between the shape and the mesh.
    }

    if (shape) {
        if (shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE) {
            printf("NumChildShapes:		%1d\n",static_cast <btCompoundShape*>(shape)->getNumChildShapes());
            DisplayAvgNumberOfHullVerticesInChildShapes(static_cast <btCompoundShape*>(shape));
        }
        const btVector3 fullExtents = btUtils::GetAabbFullExtents(shape);
        printf("FullAabbExtents:	(%1.4f,%1.4f,%1.4f)\n",fullExtents.x(),fullExtents.y(),fullExtents.z());
                setCameraDistance(btScalar(3.)*fullExtents.z());

        btScalar yPos = fullExtents.y();
        if (params.decomposeACenteredCopyOfTheMesh) yPos*=0.5f;
        btTransform T; T.setIdentity(); T.setOrigin(btVector3(0,yPos,0));
        if (shape) localCreateRigidBody(mass,T,shape);

        // Adjust the camera target:
        m_cameraTargetPosition[0] = 0;
        m_cameraTargetPosition[1] = fullExtents.y()*btScalar(0.5);
        m_cameraTargetPosition[2] = 0;
    }

    if (shape) createGround(btUtils::GetAabbHalfExtents(shape).z()>100 ?  btUtils::GetAabbHalfExtents(shape).z() : 100);
    else createGround();


    // Assign the mesh to the shape, so that it is displayed in one of the rendering modes ('r' key)
        if (meshLoadedFromStridingMeshInterface) {
            for (int i=0;i<mesh.getNumMeshSubParts();i++) mesh.setMeshSubPartMaterial(i,materials[i%numMaterials]);
            this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
        }
        else    {
            bool useMaterialOverride = mesh.getNumMeshSubParts()==1 && mesh.getMeshSubPart(0)->textureId==0;
            if (useMaterialOverride)	{
                const Material& mat = *(mesh.getMeshSubPartMaterial(0));
                if (!(mat.dif[0]==mat.dif[1] && mat.dif[1]==mat.dif[2]))  useMaterialOverride = false;
            }
            if (!useMaterialOverride) this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape);
            else {
                this->AssignMesh(shape,mesh,extraParamUseFlatNormalsWhenAssigningAMeshToTheDecomposedShape,btVector3(1,1,1),materials[0]); // If we don't want flat normals, we can use 'false' here
            }
        }

    printf("\nPress '[' or ']' to switch to another demo.\nPress 'c' to change the color mode.\nPress 'r' to change the render mode.\nPress 'g' to toggle shadows on and off.\nPress 'f' to toggle cull face mode.\nPress 'a' to toggle free camera mode (and use LMB to look around).\nPress 'o' to toggle ortho mode.\nPress 'R' to show/hide raycast screenshot.\nUse RMB to grab and MMB to shoot a box.\nAnd please do it while the OpenGL window has focus.\n\n");

}
#endif //EXCLUDE_VHACD


void	HACDDemo::clientResetScene()
{
	exitPhysics();
	initPhysics();
}
	

void	HACDDemo::keyboardCallback(unsigned char key, int x, int y)
{
	switch(key)
	{
	case	']':	++m_demoNumber;if (m_demoNumber>=numDemos) m_demoNumber=0;clientResetScene();break;
	case	'[':	--m_demoNumber;if (m_demoNumber<0) m_demoNumber=numDemos-1;clientResetScene();break;
	case	'c':	++g_colorMode;if(g_colorMode==2) g_colorMode=0;break;
	case 	'r':	{
					++g_renderMode;
					if (g_renderMode==3) g_renderMode=0;
					#ifdef SKIP_BTSHAPEHULL_RENDERMODE
					if (g_renderMode==0) ++g_renderMode;
					#endif //SKIP_BTSHAPEHULL_RENDERMODE
	}
	break;
	case	'f':	g_cullBackFaces=!g_cullBackFaces;break;	//This probably works only without shadows ('g' key)
	default:		GlutDemoApplicationEx::keyboardCallback(key,x,y);
	}
}


void	HACDDemo::exitPhysics()
{
	// It's safer for exitPhysics() if it's possible to call it multiple times without any damage.
	
	GlutDemoApplicationEx::deleteDynamicsWorld();	// Use this for worlds created with HACDDemo::createEmptyDynamicsWorld()


	for (int i=0;i<gPersistentMeshes.size();i++)	{
		btTriangleMeshEx* tme = gPersistentMeshes[i];
		if (tme) delete tme;
	}	
	gPersistentMeshes.clear();

	gContactAddedCallback = NULL;
	
	gSledgeShape = NULL;
	gSledgeShapeSubmeshIndexOfChildShapes.clear();

}

void DisplayAvgNumberOfHullVerticesInChildShapes(const btCompoundShape* cs)	{
	if (!cs || cs->getShapeType()!=COMPOUND_SHAPE_PROXYTYPE) return;
	int numConvexHullChildShapes = 0;
	int sumConvexHullVertices = 0;
	for (int i=0,sz=cs->getNumChildShapes();i<sz;i++)	{
		const btCollisionShape* childShape = cs->getChildShape(i);
		if (!childShape || childShape->getShapeType()!=CONVEX_HULL_SHAPE_PROXYTYPE) continue;
		++numConvexHullChildShapes;
		const btConvexHullShape* chs = static_cast < const btConvexHullShape* > (childShape);
		sumConvexHullVertices+= chs->getNumPoints();
	}
	if (numConvexHullChildShapes == 0) return;
	printf("AvgConvexHullVerticesInChildShapes:	%1.2f	{",(float)sumConvexHullVertices/(float)numConvexHullChildShapes);
	int numHullVerticesBiggerThan42 = 0;
	for (int i=0,sz=cs->getNumChildShapes();i<sz;i++)	{
		const btCollisionShape* childShape = cs->getChildShape(i);
		if (!childShape || childShape->getShapeType()!=CONVEX_HULL_SHAPE_PROXYTYPE) {
			printf("0");
			if (i!=sz-1) printf(",");
			continue;
		}	
		const btConvexHullShape* chs = static_cast < const btConvexHullShape* > (childShape);
		if (chs->getNumPoints()>42) ++numHullVerticesBiggerThan42;
		printf("%1d",chs->getNumPoints());
		if (i!=sz-1) printf(",");
	}	
	printf("}\n");
}



