/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
#ifndef BASIC_DEMO_H
#define BASIC_DEMO_H

// 'r' key (g_renderMode) can have 3 modes: 'btShapeHull' (Bullet default), 'btConvexHullComputer' and 'assigned mesh' (if available)
#define SKIP_BTSHAPEHULL_RENDERMODE 

//#define EXCLUDE_VHACD //Twickable

#include "GlutDemoApplicationEx.h"
#include "LinearMath/btAlignedObjectArray.h"

class btBroadphaseInterface;
class btCollisionShape;
class btOverlappingPairCache;
class btCollisionDispatcher;
class btConstraintSolver;
struct btCollisionAlgorithmCreateFunc;
class btDefaultCollisionConfiguration;


class HACDDemo : public GlutDemoApplicationEx
{

	public:
	HACDDemo()	
	:	m_demoNumber(-1)
	{
		#ifdef SKIP_BTSHAPEHULL_RENDERMODE
		g_renderMode = 1;	// 'r' key modes: g_renderMode=0 is the 'btShapeHull' render mode (Bullet default).
		#endif //SKIP_BTSHAPEHULL_RENDERMODE	
		g_colorMode = 1;	// start with a different color for each child shape.			

	}
	virtual ~HACDDemo(){
		exitPhysics();
	}
	
    virtual void myinit();  // Optional: DemoApplication::myinit() it's the InitGL() method...

	void 	createEmptyDynamicsWorld();

	void 	createGround(btScalar groundHalfDimension = 100);

	void	initPhysics();
	
	
	void	initPhysicsForDemoBunnyFamily();
	
	void	initPhysicsForDemoTorusEvolution();

	void	initPhysicsForDemoTablesAndChairs();

	void	initPhysicsForDemoSlidingSledge();

    #ifndef EXCLUDE_VHACD
    void	initPhysicsForDemoVHACDTest();
    void	initPhysicForCommandlineUsageVHACD();
    #endif


	void	initPhysicForCommandlineUsage();

	void	exitPhysics();

	virtual void clientMoveAndDisplay();

	virtual void displayCallback();
	virtual void	clientResetScene();
	

	protected:
	int m_demoNumber;
    #ifdef EXCLUDE_VHACD
	static const int numDemos = 4;
    #else
    static const int numDemos = 5;
    #endif

	void	keyboardCallback(unsigned char key, int x, int y);
};


#endif //BASIC_DEMO_H

